@@ -38,7 +38,7 @@ impl Page {
 
     /// html selector for valid web pages for domain
     pub fn get_page_selectors(&self, domain: &str) -> Selector {
-        let media_ignore_selector = r#":not([href$=".png"]):not([href$=".jpg"]):not([href$=".mp4"]):not([href$=".mp3"]):not([href$=".gif"])"#;
+        let media_ignore_selector = r#":not([href$=".png"]):not([href$=".jpg"]):not([href$=".mp4"]):not([href$=".mp3"]):not([href$=".gif"]):not([href$=".pdf"])"#;
         let relative_selector = &format!(
             r#"a[href^="/"]{}"#,
             media_ignore_selector,
@@ -63,20 +63,13 @@ impl Page {
     }
 
     /// Find all href links and return them
-    pub fn links(&self, domain: &str) -> Vec<String> {
-        let mut urls: Vec<String> = Vec::new();
-        let selector = self.get_page_selectors(&domain);
-
+    pub fn links(&self) -> Vec<String> {
+        let selector = self.get_page_selectors(&self.url);
         let html = self.get_html();
-        let anchors = html.select(&selector);
 
-        for anchor in anchors {
-            if let Some(href) = anchor.value().attr("href") {
-                urls.push(self.abs_path(href).to_string());
-            }
-        }
-
-        urls
+        html.select(&selector)
+            .map(|a| self.abs_path(a.value().attr("href").unwrap_or_default()).to_string())
+            .collect()
     }
 
     fn abs_path(&self, href: &str) -> Url {
@@ -98,15 +91,15 @@ fn parse_links() {
         .unwrap();
 
     let link_result = "https://choosealicense.com/";
-    let html = fetch_page_html(&link_result, &client).unwrap();
+    let html = fetch_page_html(&link_result, &client);
     let page: Page = Page::new(&link_result, &html);
 
     assert!(
-        page.links("https://choosealicense.com")
+        page.links()
             .contains(&"https://choosealicense.com/about/".to_string()),
         "Could not find {}. Theses URLs was found {:?}",
         page.url,
-        page.links("https://choosealicense.com")
+        page.links()
     );
 }
 
@@ -118,7 +111,7 @@ fn test_abs_path() {
         .build()
         .unwrap();
     let link_result = "https://choosealicense.com/";
-    let html = fetch_page_html(&link_result, &client).unwrap();
+    let html = fetch_page_html(&link_result, &client);
     let page: Page = Page::new(&link_result, &html);
 
     assert_eq!(
