@@ -9,6 +9,8 @@ use robotparser_fork::RobotFileParser;
 
 use std::collections::HashSet;
 use std::{sync, thread, time::Duration};
+use reqwest::header::CONNECTION;
+use reqwest::header;
 
 /// Represent a website to scrawl. To start crawling, instanciate a new `struct` using
 /// <pre>
@@ -26,7 +28,7 @@ pub struct Website<'a> {
     // configuration properies
     pub configuration: Configuration,
     /// this is a start URL given when instanciate with `new`
-    domain: String,
+    pub domain: String,
     /// contains all non-visited URL
     links: HashSet<String>,
     /// contains all visited URL
@@ -37,25 +39,22 @@ pub struct Website<'a> {
     pub on_link_find_callback: fn(String) -> String,
     /// Robot.txt parser holder
     robot_file_parser: RobotFileParser<'a>,
-    // fetch client
-    client: Client,
+    // ignore holding page in memory, pages will always be empty
+    pub page_store_ignore: bool,
 }
 
 impl<'a> Website<'a> {
     /// Initialize Website object with a start link to scrawl.
     pub fn new(domain: &str) -> Self {
-        // create home link
-        let links: HashSet<String> = vec![format!("{}/", domain)].into_iter().collect();
-
         Self {
             configuration: Configuration::new(),
-            domain: domain.to_string(),
-            links,
             links_visited: HashSet::new(),
             pages: Vec::new(),
             robot_file_parser: RobotFileParser::new(&format!("{}/robots.txt", domain)), // TODO: lazy establish
+            links: HashSet::from([format!("{}/", domain)]),
             on_link_find_callback: |s| s,
-            client: Client::new(),
+            page_store_ignore: false,
+            domain: domain.to_owned(),
         }
     }
 
@@ -83,10 +82,13 @@ impl<'a> Website<'a> {
     }
 
     /// configure http client
-    pub fn configure_http_client(&mut self, user_agent: Option<String>) {
-        self.client = Client::builder()
+    fn configure_http_client(&mut self, user_agent: Option<String>) -> Client {
+        let mut headers = header::HeaderMap::new();
+        headers.insert(CONNECTION, header::HeaderValue::from_static("keep-alive"));
+
+        Client::builder()
+            .default_headers(headers)
             .user_agent(user_agent.unwrap_or(self.configuration.user_agent.to_string()))
-            .pool_max_idle_per_host(0)
             .build()
             .expect("Failed building client.")
     }
@@ -102,14 +104,13 @@ impl<'a> Website<'a> {
     /// Start to crawl website
     pub fn crawl(&mut self) {
         self.configure_robots_parser();
-        self.configure_http_client(None);
+        let client = self.configure_http_client(None);
         let delay = self.get_delay();
-        let pool = self.create_thread_pool();
         let on_link_find_callback = self.on_link_find_callback;
-
+        let pool = self.create_thread_pool();
+        
         // crawl while links exists
         while !self.links.is_empty() {
-            let mut new_links: HashSet<String> = HashSet::new();
             let (tx, rx) = sync::mpsc::channel();
 
             for link in self.links.iter() {
@@ -117,26 +118,39 @@ impl<'a> Website<'a> {
                     continue;
                 }
                 self.log(&format!("- fetch {}", &link));
-                let thread_link = link.to_string();
+                self.links_visited.insert(String::from(link));
+
+                let link = link.clone();
                 let tx = tx.clone();
-                let cx = self.client.clone();
+                let cx = client.clone();
 
                 pool.spawn(move || {
-                    let link_result = on_link_find_callback(thread_link);
-                    let html = fetch_page_html(&link_result, &cx).unwrap_or_default();
-                    tx.send(Page::new(&link_result, &html)).unwrap();
-                    thread::sleep(delay);
+                    let link_result = on_link_find_callback(link);
+                    let html = fetch_page_html(&link_result, &cx);
+                    let page = Page::new(&link_result, &html);
+                    let links = page.links();
+
+                    tx.send((page, links)).unwrap();
                 });
             }
 
             drop(tx);
 
+            let mut new_links: HashSet<String> = HashSet::new();
+
             rx.into_iter().for_each(|page| {
-                let url = page.get_url();
-                self.log(&format!("- parse {}", url));
-                new_links.extend(page.links(&self.domain));
-                self.links_visited.insert(url.to_string());
-                self.pages.push(page);
+                let (page, links) = page;
+                self.log(&format!("- parse {}", page.get_url()));
+
+                new_links.extend(links);
+
+                if !self.page_store_ignore {
+                    self.pages.push(page);
+                }
+
+                if self.configuration.delay > 0 {
+                    thread::sleep(delay);
+                }
             });
 
             self.links = new_links;
@@ -152,11 +166,9 @@ impl<'a> Website<'a> {
         if self.links_visited.contains(link) {
             return false;
         }
-
         if contains(&self.configuration.blacklist_url, link) {
             return false;
         }
-
         if self.configuration.respect_robots_txt && !self.robot_file_parser.can_fetch("*", link) {
             return false;
         }
